import org.ietf.jgss.GSSException;
import org.ietf.jgss.Oid;

import com.sun.security.auth.module.Krb5LoginModule;
import javax.security.auth.Subject;
import javax.security.auth.login.AppConfigurationEntry;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.security.auth.login.Configuration;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class Krb5Client {

    private static Oid krb5Oid;
    private Subject mSubject;

    public static void main(String[] args) {
        try {
            krb5Oid = new Oid("1.2.840.113554.1.2.2");
        } catch (GSSException pE) {
            System.err.println("Main --> Error obtaining Kerberos V5 OID: " + pE);
            pE.printStackTrace();
            System.exit(-1);
        }
        System.setProperty("sun.security.krb5.debug", "false");
        javax.security.auth.login.Configuration.setConfiguration(createJaasConfig());
        System.setProperty("javax.security.auth.useSubjectCredsOnly", "true");
        // Update the following with the domain name, e.g. "OCBC.COM"
        // System.setProperty("java.security.krb5.realm", "BIGTOM.LOCAL");
        System.setProperty("java.security.krb5.realm", "BANKGROUP.BANK.COM");
        // System.setProperty("java.security.krb5.kdc", "deb-openldap.bigtom.local");
        System.setProperty("java.security.krb5.kdc", "kdc-server.bigtom.local");

        Krb5Client client = new Krb5Client();
        // Update the following line with the username, password
        // client.login("thomas-pk", "password");
        client.login("trader", "trader");

        System.out.println("Main --> Completed all authentication activities.");
    }

    private void login(String pUsername, String pPassword) {
        LoginContext loginContext = null;
        try {
            loginContext = new LoginContext("Client", new LoginCallbackHandler(pUsername, pPassword));
            loginContext.login();
        } catch (LoginException pE) {
            // Reference URL: https://stackoverflow.com/questions/9738756/jaas-kerberos-login-exception-on-wrong-username-password
            System.err.println("An error occured during login.");
            Throwable t = pE.getCause();
            if (t instanceof IOException) {
                System.err.println( "Network issue");
                System.exit(-2);
            } else if (t != null) {
                System.err.println(t.getClass());
                System.exit(-4);
            }            
            if (pE.toString().contains("PREAUTH_FAILED")) {
                System.err.println("Main --> Invalid username or password.");
                System.exit(-5);
            }
            System.err.println("Main --> Error creating Login Context. " + pE);
            pE.printStackTrace();
            System.exit(-6);
        }
        mSubject = loginContext.getSubject();
    }

    private static Configuration createJaasConfig() {
        Map<String, Object> mutableOptions = new HashMap<>();
        mutableOptions.put("userFirstPass", "false");
        mutableOptions.put("debug", "true");
        mutableOptions.put("useKeyTab", "false");
        mutableOptions.put("useTicketCache", "true");
        Map<String, Object> options = Collections.unmodifiableMap(mutableOptions);

        // Create entries.
        AppConfigurationEntry[] entries = {
                new AppConfigurationEntry(
                        Krb5LoginModule.class.getCanonicalName(),
                        AppConfigurationEntry.LoginModuleControlFlag.REQUIRED,
                        options)
        };

        // Create configuration.
        return new Configuration() {
            @Override
            public AppConfigurationEntry[] getAppConfigurationEntry(String name) {
                // checkArgument(JAAS_CONFIG_NAME.equals(name));
                return entries;
            }
        };
    }
}
